
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "Eo.h"
#include "Evas.h"

#include "Eo.hh"
#include "Eina.hh"
#include "Evas.hh"

#include "main.h"

#define EXPEDITE_CXX_TEST_IMPL
#include "image_data_argb_alpha_capi.h"

/* standard var */
static int done = 0;

/* private data */
static efl::eina::list<evas::image> images;

/* setup */
static void _setup(void)
{
   evas::canvas canvas(::eo_ref(G_evas));
   for (int i = 0; i < 1; i++)
     {
        evas::image o(efl::eo::parent = canvas);
        images.push_back(o);
        o.content_hint_set(EVAS_IMAGE_CONTENT_HINT_DYNAMIC);
        o.colorspace_set(EVAS_COLORSPACE_ARGB8888);
        o.size_set(640, 480);
        o.alpha_set(1);
        o.fill_set(0, 0, 640, 480);
        o.evas::object::size_set(640, 480);
        o.visibility_set(true);
     }
   done = 0;
}

/* cleanup */
static void _cleanup(void)
{
   for (evas::image& i : images)
     i.parent_set(efl::eo::base(nullptr));
   images.clear();
}

/* loop - do things */
static void _loop(double t, int f)
{
   for (evas::image& img : images)
     {
        int a, r, g, b;
        Evas_Coord w = 640;
        Evas_Coord h = 480;
        Evas_Coord x = (win_w / 2) - (w / 2);
        Evas_Coord y = (win_h / 2) - (h / 2);
        img.evas::object::position_set(x, y);
        img.evas::object::size_set(w, h);
        img.fill_set(0, 0, w, h);
        unsigned int *data = reinterpret_cast<unsigned int*>(img.data_get(1));
        int st = img.stride_get() >> 2;
        unsigned int *p = data;
        for (y = 0; y < h; y++)
          {
             for (x = 0; x < w; x++)
               {
                  r = (x * y / 7) + f;
                  g = (x / 2);
                  b = (y / 2);
                  a = (x + y);
                  r &= 0xff;
                  g &= 0xff;
                  b &= 0xff;
                  a &= 0xff;
                  r = (a * r) / 255;
                  g = (a * g) / 255;
                  b = (a * b) / 255;
                  *p = (a << 24) | (r << 16) | (g << 8) | b;
                  p++;
               }
             p += (st - w);
          }
        img.data_set(data);
        img.data_update_add( 0, 0, w, h);
     }
   FPS_STD(NAME);
}

/* prepend special key handlers if interactive (before STD) */
static void _key(char *key)
{
   KEY_STD;
}

extern "C" void FNAME(void)
{
   _setup();
   ui_func_set(_key, _loop);
}
