
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "Eo.h"
#include "Evas.h"

#include "Eo.hh"
#include "Eina.hh"
#include "Evas.hh"

#include "main.h"

#define EXPEDITE_CXX_TEST_IMPL
#include "image_data_ycbcr601pl_wide_stride_capi.h"

/* standard var */
static int done = 0;

/* private data */
static efl::eina::list<evas::image> images;
static unsigned char *yp = NULL, *up = NULL, *vp = NULL;

/* setup */
static void _setup(void)
{
   evas::canvas canvas(::eo_ref(G_evas));
   for (int i = 0; i < 1; i++)
     {
        evas::image o(efl::eo::parent = canvas);
        images.push_back(o);
        o.content_hint_set(EVAS_IMAGE_CONTENT_HINT_DYNAMIC);
        o.colorspace_set(EVAS_COLORSPACE_YCBCR422P601_PL);
        o.size_set(320, 480);
        o.alpha_set(0);
        o.fill_set(0, 0, 640, 480);
        o.evas::object::size_set(640, 480);
        o.evas::object::visibility_set(true);
        yp = new unsigned char[640 * 480];
        up = new unsigned char[320 * 240];
        vp = new unsigned char[320 * 240];
        FILE *f = fopen(build_path("tp.yuv"), "rb");
        if (f)
          {
             fread(yp, 640 * 480, 1, f);
             fread(up, 320 * 240, 1, f);
             fread(vp, 320 * 240, 1, f);
             fclose(f);
          }
        unsigned char **data = static_cast<unsigned char**>(o.data_get(1));
        if (data)
          {
             unsigned char **lp = data;
             for (int y = 0; y < 480; y++)
               {
                  *lp = yp + (y * 640);
                  lp++;
               }
             for (int y = 0; y < 240; y++)
               {
                  *lp = up + (y * 320);
                  lp++;
               }
             for (int y = 0; y < 240; y++)
               {
                  *lp = vp + (y * 320);
                  lp++;
               }
             o.data_set(data);
             o.data_update_add( 0, 0, 320, 480);
          }
     }
   done = 0;
}

/* cleanup */
static void _cleanup(void)
{
   for (evas::image& i : images)
     i.parent_set(efl::eo::base(nullptr));
   images.clear();

   delete yp;
   delete up;
   delete vp;
}

/* loop - do things */
static void _loop(double t, int f)
{
   for (evas::image& img : images)
     {
        Evas_Coord w = 640;
	Evas_Coord h = 480;
	Evas_Coord x = (win_w / 2) - (w / 2);
	Evas_Coord y = (win_h / 2) - (h / 2);
        img.evas::object::position_set(x, y);
        img.evas::object::size_set(w, h);
        img.fill_set(0, 0, w, h);

	w = 320 - 16 + f;
	if (w < 1) w = 1;
	else if (w > 640) w = 640;

	w &= ~0x1;

        img.size_set(w, 480);
        unsigned char **data = static_cast<unsigned char**>(img.data_get(1));
	if (data)
	  {
	     unsigned char **lp = data;
	     for (int y = 0; y < 480; y++)
	       {
		  *lp = yp + (y * 640);
		  lp++;
	       }
	     for (int y = 0; y < 240; y++)
	       {
		  *lp = up + (y * 320);
		  lp++;
	       }
	     for (int y = 0; y < 240; y++)
	       {
		  *lp = vp + (y * 320);
		  lp++;
	       }
             img.data_set(data);
             img.data_update_add(0, 0, w, 480);
	  }
     }
   FPS_STD(NAME);
}

/* prepend special key handlers if interactive (before STD) */
static void _key(char *key)
{
   KEY_STD;
}

extern "C" void FNAME(void)
{
   ui_func_set(_key, _loop);
   _setup();
}
