
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "Eo.h"
#include "Evas.h"

#include "Eo.hh"
#include "Eina.hh"
#include "Evas.hh"

#include "main.h"

#define EXPEDITE_CXX_TEST_IMPL
#include "image_map_3d_5_capi.h"

typedef struct _Point
{
   Evas_Coord x, y, z, u, v;
} Point;

typedef struct _Side
{
   evas::image *o;
   Point pt[4];
} Side;

typedef struct _Cube
{
   Side side[6];
} Cube;

/* standard var */
static int done = 0;

/* private data */
static Cube *cubes[1];

#define POINT(n, p, xx, yy, zz, uu, vv) \
   c->side[n].pt[p].x = xx; \
   c->side[n].pt[p].y = yy; \
   c->side[n].pt[p].z = zz; \
   c->side[n].pt[p].u = uu; \
   c->side[n].pt[p].v = vv

static Cube *
_cube_new(Evas_Coord w, Evas_Coord h, Evas_Coord d)
{
   evas::canvas canvas(::eo_ref(G_evas));
   w -= (w / 2);
   h -= (h / 2);
   d -= (d / 2);
   Cube *c = new Cube {};
   for (int i = 0; i < 6; i++)
     {
        char buf[256];
        c->side[i].o = new evas::image(efl::eo::parent = canvas);
        snprintf(buf, sizeof(buf), "im%i.png", (i % 2) + 1);
        eo_do(c->side[i].o->_eo_ptr(), efl_file_set(build_path(buf), NULL)); // XXX
        c->side[i].o->fill_set(0, 0, 720, 420);
        c->side[i].o->evas::object::size_set(720, 420);
        c->side[i].o->smooth_scale_set(0);
        c->side[i].o->visibility_set(true);
     }
   POINT(0, 0, -w, -h, -d,   0,   0);
   POINT(0, 1,  w, -h, -d, 720,   0);
   POINT(0, 2,  w,  h, -d, 720, 420);
   POINT(0, 3, -w,  h, -d,   0, 420);

   POINT(1, 0,  w, -h, -d,   0,   0);
   POINT(1, 1,  w, -h,  d, 720,   0);
   POINT(1, 2,  w,  h,  d, 720, 420);
   POINT(1, 3,  w,  h, -d,   0, 420);

   POINT(2, 0,  w, -h,  d,   0,   0);
   POINT(2, 1, -w, -h,  d, 720,   0);
   POINT(2, 2, -w,  h,  d, 720, 420);
   POINT(2, 3,  w,  h,  d,   0, 420);

   POINT(3, 0, -w, -h,  d,   0,   0);
   POINT(3, 1, -w, -h, -d, 720,   0);
   POINT(3, 2, -w,  h, -d, 720, 420);
   POINT(3, 3, -w,  h,  d,   0, 420);

   POINT(4, 0, -w, -h,  d,   0,   0);
   POINT(4, 1,  w, -h,  d, 720,   0);
   POINT(4, 2,  w, -h, -d, 720, 420);
   POINT(4, 3, -w, -h, -d,   0, 420);

   POINT(5, 0, -w,  h, -d,   0,   0);
   POINT(5, 1,  w,  h, -d, 720,   0);
   POINT(5, 2,  w,  h,  d, 720, 420);
   POINT(5, 3, -w,  h,  d,   0, 420);

   return c;
}

static void
_cube_pos(Cube *c,
          Evas_Coord x, Evas_Coord y, Evas_Coord z,
          double dx, double dy, double dz)
{
   static Evas_Map *m = NULL;
   int i, j, order[6], sorted;
   Evas_Coord mz[6];

   if (!m) m = ::evas_map_new(4);
   ::evas_map_smooth_set(m, 0);

   for (i = 0; i < 6; i++)
     {
        Evas_Coord tz[4];

        for (j = 0; j < 4; j++)
          {
             ::evas_map_point_coord_set(m, j,
                                      c->side[i].pt[j].x + x,
                                      c->side[i].pt[j].y + y,
                                      c->side[i].pt[j].z + z);
             ::evas_map_point_image_uv_set(m, j,
                                      c->side[i].pt[j].u,
                                      c->side[i].pt[j].v);
             ::evas_map_point_color_set(m, j, 255, 255, 255, 255);
          }
        ::evas_map_util_3d_rotate(m, dx, dy, dz, x, y, z);
        ::evas_map_util_3d_lighting(m, -1000, win_h / 2, -1000,
                                  255, 255, 255,
                                  20, 20, 20);
        ::evas_map_util_3d_perspective(m, (win_w / 2), (win_h / 2), -360, 720);
        if (::evas_map_util_clockwise_get(m))
          {
             c->side[i].o->map_enable_set(1);
             c->side[i].o->map_set(m);
             c->side[i].o->visibility_set(true);
          }
        else
          {
             c->side[i].o->visibility_set(false);
          }

        order[i] = i;
        for (j = 0; j < 4; j++)
          ::evas_map_point_coord_get(m, j, NULL, NULL, &(tz[j]));
        mz[i] = (tz[0] + tz[1] + tz[2] + tz[3]) / 4;
     }
   sorted = 0;
   do
     {
        sorted = 1;
        for (i = 0; i < 5; i++)
          {
             if (mz[order[i]] > mz[order[i + 1]])
               {
                  j = order[i];
                  order[i] = order[i + 1];
                  order[i + 1] = j;
                  sorted = 0;
               }
          }
     }
   while (!sorted);

   c->side[order[0]].o->raise();
   for (i = 1; i < 6; i++)
     c->side[order[i]].o->stack_below(*c->side[order[i - 1]].o);
}

static void
_cube_free(Cube *c)
{
   for (int i = 0; i < 6; i++)
     {
        c->side[i].o->visibility_set(false); // XXX
        c->side[i].o->parent_set(efl::eo::base(nullptr));
        delete c->side[i].o;
     }
   delete c;
}

/* setup */
static void _setup(void)
{
   cubes[0] = _cube_new(720, 420, 720);

   done = 0;
}

/* cleanup */
static void _cleanup(void)
{
   _cube_free(cubes[0]);
}

/* loop - do things */
static void _loop(double t, int f)
{
   _cube_pos(cubes[0],
             (win_w / 2), (win_h / 2), 0,
             0, f, 0);
   FPS_STD(NAME);
}

/* prepend special key handlers if interactive (before STD) */
static void _key(char *key)
{
   KEY_STD;
}

extern "C" void FNAME(void)
{
   _setup();
   ui_func_set(_key, _loop);
}
