
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "Eo.h"
#include "Evas.h"

#include "Eo.hh"
#include "Eina.hh"
#include "Evas.hh"

#include "main.h"

#define EXPEDITE_CXX_TEST_IMPL
#include "image_map_color_nearest_rotate_capi.h"

/* standard var */
static int done = 0;

/* private data */
static efl::eina::list<evas::image> images;

/* setup */
static void _setup(void)
{
   evas::canvas canvas(::eo_ref(G_evas));
   for (int i = 0; i < (OBNUM / 2); i++)
     {
        evas::image o(efl::eo::parent = canvas);
        images.push_back(o);
        eo_do(o._eo_ptr(), efl_file_set(build_path("logo.png"), NULL)); // XXX
        o.fill_set(0, 0, 120, 160);
        o.evas::object::size_set(120, 160);
        o.smooth_scale_set(0);
        o.visibility_set(true);
     }
   done = 0;
}

/* cleanup */
static void _cleanup(void)
{
   for (evas::image& i : images)
     i.parent_set(efl::eo::base(nullptr));
   images.clear();
}

/* loop - do things */
static void _loop(double t, int f)
{
   static Evas_Map *m = NULL;
   if (!m) m = evas_map_new(4);
   evas_map_smooth_set(m, 0);
   int i = 0;
   for (evas::image& img : images)
     {
        Evas_Coord w = 120;
	Evas_Coord h = 160;
	Evas_Coord x = (win_w / 2) - (w / 2);
        x += sin((double)(f + (i * 13)) / (36.7 * SLOW)) * (w / 2);
	Evas_Coord y = (win_h / 2) - (h / 2);
        y += cos((double)(f + (i * 28)) / (43.8 * SLOW)) * (h / 2);

	evas_map_util_points_populate_from_geometry(m, x, y, w, h, 0);

        evas_map_point_color_set(m, 0, 255, 255, 255, 255);
        evas_map_point_color_set(m, 1, 255, 0, 0, 255);
        evas_map_point_color_set(m, 2, 0, 0, 255, 255);
        evas_map_point_color_set(m, 3, 0, 255, 0, 255);

        evas_map_util_rotate(m, f, x + (w / 2), y + (h / 2));

        evas_obj_map_enable_set(1);
        evas_obj_map_set(m);
        ++i;
     }
   FPS_STD(NAME);
}

/* prepend special key handlers if interactive (before STD) */
static void _key(char *key)
{
   KEY_STD;
}

extern "C" void FNAME(void)
{
   _setup();
   ui_func_set(_key, _loop);
}
