
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "Eo.h"
#include "Evas.h"

#include "Eo.hh"
#include "Eina.hh"
#include "Evas.hh"

#include "main.h"

#define EXPEDITE_CXX_TEST_IMPL
#include "poly_blend_capi.h"

/* standard var */
static int done = 0;
/* private data */
static efl::eina::list<evas::polygon> images;

static void
poly(evas::polygon& o, int type, Evas_Coord x, Evas_Coord y)
{
   if (o)
     {
        o.points_clear();
     }
   switch (type % 4)
     {
      case 0: /* triangle */
	if (o)
          {
             o.point_add(x + 50 , y + 0);
             o.point_add(x + 100, y + 100);
             o.point_add(x + 0  , y + 100);
          }
	break;
      case 1: /* square */
	if (o)
          {
             o.point_add(x + 0  , y + 0);
             o.point_add(x + 100, y + 0);
             o.point_add(x + 100, y + 100);
             o.point_add(x + 0  , y + 100);
          }
	break;
      case 2: /* hex */
	if (o)
          {
             o.point_add(x + 50 , y + 0);
             o.point_add(x + 100, y + 30);
             o.point_add(x + 100, y + 70);
             o.point_add(x + 50 , y + 100);
             o.point_add(x + 0  , y + 70);
             o.point_add(x + 0  , y + 30);
          }
	break;
      case 3: /* star */
	if (o)
          {
             o.point_add(x + 50 , y + 0);
             o.point_add(x + 60 , y + 40);
             o.point_add(x + 90 , y + 30);
             o.point_add(x + 70 , y + 60);
             o.point_add(x + 90 , y + 100);
             o.point_add(x + 50 , y + 70);
             o.point_add(x + 10 , y + 100);
             o.point_add(x + 30 , y + 60);
             o.point_add(x + 10 , y + 30);
             o.point_add(x + 40 , y + 40);
          }
	break;
      default:
	break;
     }
}

/* setup */
static void _setup(void)
{
   evas::canvas canvas(::eo_ref(G_evas));
   srnd();
   for (int i = 0; i < OBNUM; i++)
     {
	int r, g, b, a;
        evas::polygon o(efl::eo::parent = canvas);
	images.push_back(o);
	a = (rnd()&0xff) / 2;
	r = ((rnd()&0xff) * a) / 255;
	g = ((rnd()&0xff) * a) / 255;
	b = ((rnd()&0xff) * a) / 255;
        o.evas::object::color_set(r, g, b, a);
        o.evas::object::visibility_set(true);
	poly(o, i, 0, 0);
     }
   done = 0;
}

/* cleanup */
static void _cleanup(void)
{
   for (evas::polygon& p : images)
     p.parent_set(efl::eo::base(nullptr));
   images.clear();
}

/* loop - do things */
static void _loop(double t, int f)
{
   int i = 0;
   Evas_Coord x, y, w, h;
   for (evas::polygon& o : images)
     {
        o.evas::object::size_get(&w, &h);
	x = (win_w / 2) - (w / 2);
	x += ::sin((double)(f + (i * 13)) / (36.7 * SLOW)) * (win_w / 4);
	y = (win_h / 2) - (h / 2);
	y += ::cos((double)(f + (i * 28)) / (43.8 * SLOW)) * (win_h / 4);
        o.evas::object::position_set(x, y);
        ++i;
     }
   FPS_STD(NAME);
}

/* prepend special key handlers if interactive (before STD) */
static void _key(char *key)
{
   KEY_STD;
}

extern "C" void FNAME(void)
{
   ui_func_set(_key, _loop);
   _setup();
}
