
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "Eo.h"
#include "Evas.h"

#include "Eo.hh"
#include "Eina.hh"
#include "Evas.hh"

#include "main.h"

#define EXPEDITE_CXX_TEST_IMPL
#include "widgets_list_4_grouped_capi.h"

/* standard var */
static int done = 0;
/* private data */
#define NUM 512
#define ICON_SIZE 32
static efl::eina::list<evas::image> images;
static efl::eina::array<evas::image> icons;
static efl::eina::list<evas::text> texts;

static const char *_icons[] =
{
   "bug.png",
   "bulb.png",
   "camera.png",
   "colorbox.png",

   "e.png",
   "error.png",
   "flower.png",
   "house.png",

   "mushroom.png",
   "pulse.png",
   "typewriter.png",
   "warning.png",

   "watch.png"
};

static const char *labels[] =
{
   "Andrew",
   "Alex",
   "Amanda",
   "Arthur",
   "Astrid",
   "Avery",

   "Beethoven",
   "Billy",
   "Bob",
   "Bundy",

   "Candy",
   "Carsten",

   "Danny",
   "Dennis",
   "Dirk",
   "Doug",

   "Edmond",
   "Erik",

   "Fernando",
   "Frank",
   "Frederick",

   "Gabby",
   "George",
   "Gilroy",
   "Goodrich",
   "Gumby",
};

/* setup */
static void _setup(void)
{
   evas::canvas canvas(::eo_ref(G_evas));
   for (int i = 0; i < NUM; i++)
     {
        evas::image o1(efl::eo::parent = canvas);
        images.push_back(o1);
        o1.filled_set(1);
        o1.border_set(2, 2, 2, 2);
        eo_do(o1._eo_ptr(), efl_file_set(build_path("pan.png"), NULL)); // XXX
        o1.evas::object::size_set(win_w, ICON_SIZE);
        o1.visibility_set(true);

        evas::image o2(efl::eo::parent = canvas);
        icons.push_back(o2);
        o2.filled_set(1);
        o2.border_set(2, 2, 2, 2);
        eo_do(o2._eo_ptr(), efl_file_set(build_path(_icons[i % 13]), NULL)); // XXX
        o2.evas::object::size_set(ICON_SIZE - 8, ICON_SIZE - 8);
        o2.visibility_set(true);

        evas::text o3(efl::eo::parent = canvas);
        texts.push_back(o3);
        o3.font_set("Vera-Bold", 10);
        o3.text_set(labels[i % 26]);
        o3.evas::object::color_set(0, 0, 0, 255);
        o3.visibility_set(true);
     }

   for (evas::image& img : images)
     {
        img.raise();
     }
   for (evas::image& ico : icons)
     {
        ico.raise();
     }
   for (int i = 0; i < NUM; i++)
     {
        if (i > 13)
          {
             icons[i].stack_above(icons[i - 13]);
          }
     }
   for (evas::text& text : texts)
     {
        text.raise();
     }
   done = 0;
}

static void _cleanup()
{
   for (evas::image& i : images)
     i.parent_set(efl::eo::base(nullptr));
   images.clear();

   for (evas::image& i : icons)
     {
        i.visibility_set(false); // XXX
        i.parent_set(efl::eo::base(nullptr));
     }
   icons.clear();

   for (evas::text& t : texts)
     t.parent_set(efl::eo::base(nullptr));
   texts.clear();
}

/* loop - do things */
static void _loop(double t, int f)
{
   Evas_Coord x, y, tw, th, cent;
   x = 0;
   y = 0 - f;
   efl::eina::list<evas::image>::iterator img = images.begin();
   efl::eina::array<evas::image>::iterator ico = icons.begin();
   for (evas::text& txt : texts)
     {
        (*img++).evas::object::position_set(x, y);
        (*ico++).evas::object::position_set(x + 4, y + 4);
        txt.evas::object::size_get(&tw, &th);
        cent = (ICON_SIZE - th) / 2;
        txt.evas::object::position_set(x + 8 + ICON_SIZE + 8, y + cent);
        y += ICON_SIZE;
     }
   FPS_STD(NAME);
}

static void _key(char *key)
{
   KEY_STD;
}

extern "C" void FNAME(void)
{
   ui_func_set(_key, _loop);
   _setup();
}
